function set_constants() {
    label_title_width = 10;
    label_xsize = 152;
    page_ysize = 210;
}

function fatal(msg) {
    print "fatal error at line " NR ": " msg > "/dev/stderr";
    exit(1)
}

function label_init( name ) {
    label_name = name;
    label_count = 0;
    delete label_lines;
}

function label_set_ysize( ysize ) {
    ysize = tolower( ysize );
    if ( ysize == "big" ) label_ysize = 48.0; 
    else if( ysize == "medium") label_ysize = 38.0;
    else if( ysize == "small") label_ysize = 28.0;
    else label_ysize = $0;
    if (label_ysize < 10) fatal( sprintf ( "label too small (%i)", 
					   label_ysize ));
    label_min_lines = 1 + label_ysize / 10 - 1;
}

function label_add_line( line ) {
    label_lines[label_count++] = line;
}

function array_length( a, #)
		       _i) {
    _i = 0;
    for (e in a) _i = _i + 1;
    return _i;
}

function print_label( title, title_width, xsize, ysize, items, #)
		      _items_length, _min_lines, _i ) {
    _items_length = array_length( items );
    _min_lines = 1 + ysize / 10 - 1;

    print "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" \
	"%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%";
    print "% label: " title;
    print "\\hspace*{25mm}%";
    print "\\resizebox*{" xsize "mm}{" ysize "mm}{%";
    print "  \\begin{minipage}[c][" ysize "mm]" \
	"[c]{" 2 "mm}%";
    print "    \\rule{0.1mm}{" ysize "mm}";
    print "  \\end{minipage}%";
    print "  \\begin{minipage}[c][" ysize "mm]" \
	"[c]{" (xsize - titlewidth) "mm}%";
    print "    \\resizebox*{!}{" (ysize - 4.5) "mm}{%";
    print "      \\parbox[c]{" (xsize - titlewidth) "mm}{%"
    print "\\FI%";
    for (_i = 0; _i < _items_length - 1; _i++ ) {
	printf "        %s\\\\\n", items[_i];
    }
    printf "        %s", items[_items_length - 1];
    for (_i = _items_length; _i < _min_lines; _i++ )
	print "\\\\";
    print "      }%";
    print "    }%";
    print "  \\end{minipage}%";
    print "  \\begin{minipage}[c][" ysize "mm][c]{" title_width "mm}%";
    print "\\FT%";
    print "    \\resizebox*{" title_width "mm}{" ysize "mm}{%";
    print "      \\rotatebox{270}{%";
    print "        \\colorbox{black}{%";
    print "          \\textcolor{white}{" title "}%";
    print "        }%";
    print "      }%";
    print "    }%";
    print "  \\end{minipage}%";
    print "}\\\\";
}

function print_label_blank( title_width, xsize, ysize, #)
			   _a ) {
    delete _a;
    _a[0] = " ";
    print_label( " ", title_width, xsize, 10, _a );

}

function print_snipper_vertical( xspace, ysize ) {
    print "%%% vertical snipper %%%";
    print "\\hspace*{25mm}\\rule{0.1mm}{" ysize "mm}\\hspace*{" xspace "mm}\\rule{0.1mm}{" ysize "mm}\n";
}

function print_snipper_horizontal() {
    print "%%% vertical snipper %%%";
    print "\\vspace*{-1mm}\\rule{2cm}{0.1mm}\\hfill\\rule{2cm}{0.1mm}\\\\";
}

function page_start() {
    no_new_page = 1;
    page_used_spaced = 0;

    # vertical snippers
    print_snipper_vertical( label_xsize, 15 );

    print_label_blank( label_title_width, label_xsize, 10 );

    # horizontal snippers
    print_snipper_horizontal();
}

function page_end() {
    no_new_page = 0;

    # small black to correct small snipping errors
    print_label_blank( label_title_width, label_xsize, 10 );

    # kill space
    print "\\vfill";

    # vertical snippers
    print_snipper_vertical( label_xsize, 15 );

    print "\\newpage\n";
}

function label_print() {
    if (!label_name)
	return;

    page_used_spaced += label_ysize;
    if (page_used_spaced > page_ysize) {
	page_end();
	page_start();
	page_used_space = label_ysize;
    }

    print_label( label_name, label_title_width, label_xsize, label_ysize,
		 label_lines );

    # horizontal snippers
    print_snipper_horizontal();
}

END {
    label_print();
    page_end();
}

BEGIN {
    set_constants();
    page_start();
}

/^\# / {
    label_print();
    gsub( "^# ", "" );
    label_init( $0 );
}

/^\| / {
    gsub( "^\\| ", "" );
    label_set_ysize( $0 );
 }

/^> / {
    gsub( "^> ", "" );
    label_add_line( $0 );
} 
